// Batlship.java
// Plays Battleship Game:
//       How many moves will it take you to Sink the Battleship?

import java.io.*;
import java.util.*; // Random Numbers here
import java.awt.*;  // Graphics Stuff in here
import java.awt.event.*; // Listeners here
import java.text.*;  // for ecimalFormat

public class Batlship2 extends Frame implements ActionListener {

 static final int HEIGHT=6, // Number of Rows
                 WIDTH=8,  // Number of Columns
                 CoordWidth=75, // Box Width in pixels
                 CoordHeight=75;// Box Height in pixels

  // Define char constants for 3 possible states of an area
    static final char Emptypoint='X',
              Hitpoint=(char)178,
              Initpoint='.';

    static char[][] Board=new char[HEIGHT][WIDTH];

    Point[] Ship=new Point[3];
    boolean Win=false;
    int Moves; // Count # Moves
    Button[][] Coord=new Button[HEIGHT][WIDTH];
    Frame OverPanel; // Query for Action at end of game
    Label GameOver;  // Label stating # moves in end of game Frame

   // Default constructor
   public Batlship2()
   {// set up window for frame Batlship
    super("!!!BattleShip!!!") ;

    // Size the window to the grid with a header of CoordHeight rows
    setSize(CoordWidth*(WIDTH)+2,CoordHeight*(HEIGHT+1)+2);

    // Use a GridLayout, which will be filled with Buttons
    setLayout(new GridLayout(HEIGHT,WIDTH));

    // Create & Insert Buttons
    for (int h=0;h<HEIGHT;h++)
        for (int w=0;w<WIDTH;w++) {
            Coord[h][w]=new Button(h+","+w);
            Coord[h][w].addActionListener(this);
            Coord[h][w].setBackground(Color.black);
            add(Coord[h][w]);
        }
    // Init the board used to keep track of actions
    InitBoard(Board);

    // Initialize the Ship's Point Objects
    for (int i=0;i<3;i++)
     Ship[i]=new Point();

    PlaceShip(Ship);

    // Set up a Frame to query user at end of game
    OverPanel=new Frame();
    // Need a border layout to place the buttons
    OverPanel.setLayout(new BorderLayout());
    Button Another=new Button("Play Again"),
    Quit=new Button("Quit");

    // Set up the Label. Need to set its text when game ends
    GameOver=new Label("                               ");
    // Add listener for Quit and Play Again buttons
    Another.addActionListener(this);
    Quit.addActionListener(this);
    // Place the Buttons on the frame
    OverPanel.add(Another,"Center");
    OverPanel.add(Quit,"South");
    // Place label message in frame
    OverPanel.add("North",GameOver);
   }// end of constructor method

   // Retrieve the indices of the pushed Button
   private Point getCoords(String CoordString)
   {StringTokenizer Tokenizer=new StringTokenizer(CoordString,",");
    return(new Point(Integer.valueOf(Tokenizer.nextToken()).intValue(),
                        Integer.valueOf(Tokenizer.nextToken()).intValue()));
   }

   // Handle pushing of buttons (can also be used for other events
   public void actionPerformed(ActionEvent Event)
   {String arg=Event.getActionCommand();
    // Deal with "Play Again" button
    if ("Play Again".equals(arg)) {
      // Reinitialize the game
      InitBoard(Board); // Must clear the board
      Moves=0;          // Reset # Moves
      Win=false;        // Otherwise, will not play next game
      OverPanel.setVisible(false); // Remove End of Game queries
      PlaceShip(Ship);  // New location for the ship
      // Reset Buttons
      for (int h=0;h<HEIGHT;h++)
        for (int w=0;w<WIDTH;w++) {
            Coord[h][w].addActionListener(this);
            Coord[h][w].setBackground(Color.black);
            Coord[h][w].setForeground(Color.black);
            Coord[h][w].setLabel(h+","+w);
        }
    }
    else if ("Quit".equals(arg)) {
     // Deal with "Quit" Button
      dispose(); // Get rid of Batlship frame
      System.exit(0);
    }
    else { // Button pressed in game board
     // Get the Button pressed
      Point Spot=getCoords(arg);
      boolean ok=(!(Spot.x < 0 || Spot.x >= HEIGHT
                 || Spot.y < 0 || Spot.y >=WIDTH)
                       && !Win && pointStatus(Spot,Board) ==Initpoint);
      if (ok) {
       Moves++;
       // Change the button: Remove the listener, since each Button can
       //        only be pressed once; change its color and replace its text
       if (Hit(Spot,Ship)) {
        Board[Spot.x][Spot.y]=Hitpoint;     // put box there
        Coord[Spot.x][Spot.y].setForeground(Color.red);
        Coord[Spot.x][Spot.y].setLabel("HIT");
       }
       else {
        Board[Spot.x][Spot.y]=Emptypoint;       // put box there
        Coord[Spot.x][Spot.y].setForeground(Color.white);
        Coord[Spot.x][Spot.y].setLabel("MISS");
       }
       Coord[Spot.x][Spot.y].removeActionListener(this);
       if (Win=GameWon(Board,Ship)) {
        GameOver.setText("Battleship Sunk in "+Moves+" Moves");
        OverPanel.pack(); // Condense frame to size required
        OverPanel.setVisible(true); // Display the end of game query frame
       } // if Win
      } // if ok
    } // else
   } // end of method actionPerformed

   // Initialize the board to reflect that no areas have been selected
   static void InitBoard(char Board[][])
   {int x,y;
    for(y=0; y<HEIGHT; y++)        // fill Board with periods
     for(x=0; x<WIDTH; x++)
      Board[y][x] = Initpoint;
   }

   // Compare two points (Should this be in the point class?)
   // Returns: true==>Points equal  false otherwise
   static boolean pointsEqual(Point X,Point Y)
   {return(X.x==Y.x && X.y==Y.y);}

   // Return the char stored in the point represented by P
   static char pointStatus(Point P,char Grid[][])
   {return(Grid[P.x][P.y]);}

   // Determine if P is the same as one of the points in Ship
   // true==>Hit  Miss, otherwise
   static boolean Hit(Point P,Point Boat[])
   {int i;
    for (i=0;i<3 && !pointsEqual(P,Boat[i]);i++);
    return(i<3);
   }

   // Place a ship in the grid, which is 0..HEIGHT-1 0..WIDTH-1
   // The first point and the direction (vertical or horizontal)
   //     are chosen randomly
   static void PlaceShip(Point Ship[])
   {int Direction;
    Random Spots=new Random();
    Ship[0].x=(int)(Spots.nextDouble()*3);  // Ship starts in 1st 3 rows
    Ship[0].y=(int)(Spots.nextDouble()*6);  // Ship starts in 1st 6 columns
    Direction=(int)(Spots.nextDouble()*2);  // 0 ==> Vertical  1 ==> Horizontal
    if (Direction==1) {
     Ship[1].x=Ship[2].x=Ship[0].x;
     Ship[1].y=Ship[0].y+1;
     Ship[2].y=Ship[1].y+1;
    }
    else {
     Ship[1].y=Ship[2].y=Ship[0].y;
     Ship[1].x=Ship[0].x+1;
     Ship[2].x=Ship[1].x+1;
    }
   }

   // Determine if all 3 points in the ship have been clicked
   static boolean GameWon(char Board[][],Point Ship[])
   {boolean Done=true;
    for (int i=0;i<3 && Done;i++)
     if (Board[Ship[i].x][Ship[i].y]!=Hitpoint) Done=false;
    return(Done);
   }

   public static void main(String[] args) throws java.io.IOException
   {Batlship2 SeaWolf=new Batlship2();
    // Make the Frame Visible
    SeaWolf.setVisible(true);
   } // end of main

} // End of Class

